@@ -24,15 +24,38 @@ c         initialize
       real izmin     ! input. xmin. not in log even log10
       real izbinORzmax      ! input. bin.  If log, bin is for log10
       integer iztklg    ! input.   same as ixtklg 
+      integer xstep, ystep ! input
+
       integer dealloc
-      real*8 sum 
-      integer   xnbin, ynbin, znbin
+      real sum 
+      character*64 item(4)
+      integer nitem
+      integer nc 
+      real normf
+      integer*2   xnbin, ynbin, znbin
                        !         
       character*(*)  id
+      character*(*)  title      !  input. graph title
+      character*(*)  categ      !  input. max 8 char. category of histogram suc as lateral
+      real pw                   !  input. vertical scale is displayed by x**pw dN/dx
+                                !  in default
+      logical logv              !  input. if T, vertical scale is displayed in
+                                !  log in default
+      character*(*)  labelx     !  input. a few char string for x-axis
+                                !   (must not include special char).
+      character*(*)  unitx      !  input. a few char string for x-axis unit
+      character*(*)  labely     !  input. same as
+      character*(*)  unity      !  input.    above.
+      character*(*)  labelz     !  input. same as
+      character*(*)  unitz      !  input.    above.
+      character*(*)  dir        ! input.  
+      character*(*)  dNunit ! input.
+
       integer klena
 c     ******************
-      include 'Z90histc.f'
-      include "Z90hist3.f"
+      include 'Z90histc.h'
+      include 'Z90histo.h'
+      include "Z90hist3.h"
       type(histogram3) h, h1, h2
 c     ====================      
       real inorm   !  input. used in the normalization as dN/dx/inorm
@@ -45,8 +68,13 @@ c     ====================
       integer bfnor  ! input. binary  read file no.
       integer icon   ! output. 0; binary read was successful
                      !  1;unexpected EOF
-                  
+      integer evno   ! input
 c
+      integer  itempz, itempv
+      integer istep
+      character*96 dirstr
+      character*32 filen
+      integer nchar
       real  x, y, z,  w 
       real  xx, yy, zz
       integer  i, j, k, ndiv
@@ -55,20 +83,21 @@ c
 
 
 
-      if( h%init .eq. 'initend') then
-         write(0, *) ' already instanciated; xmin=',ixmin,
-     *    ' binORmax=',ixbinORxmax, ' inbin=', ixnbin, ' itklg=',ixtklg
-         write(0, *) '  ymin=',iymin,
-     * ' binORmax=',iybinORymax, ' iynbin=', iynbin, ' itklg=',iytklg
-         write(0, *) '  zmin=',izmin,
-     * ' binORmax=',izbinORzmax, ' iznbin=', iznbin, ' itklg=',iztklg
+      if( h%c%init .eq. 'initend') then
+         write(0, *) '3D hist.  already instanciated;'
+         write(0, *) ' title=',h%c%title
+         write(0, *) ' category=',h%c%categ
+         write(0, *) ' id=',h%c%id
+         stop 9999
       else
-         h%init = 'initend'
+         h%c%init = 'initend'
       endif
                                                                               
       h%x%nhist = ixnbin
       h%y%nhist = iynbin
       h%z%nhist = iznbin
+      h%x%step = 1
+      h%y%step = 1
       allocate( h%dnw(ixnbin, iynbin, iznbin) )
       allocate( h%dndxdydz(ixnbin, iynbin, iznbin) )
 
@@ -198,6 +227,19 @@ c
          h%z%xm = h%z%xmin - h%z%cent * h%z%bin/2.
          h%z%inc  = h%z%bin
       endif
+
+      h%c%eventno = 1
+      h%x%label =' '
+      h%x%unit = ' '
+      h%y%label =' '
+      h%y%unit = ' '
+      h%z%label =' '
+      h%z%unit = ' '
+      h%c%title = ' '
+      h%c%categ = ' '
+      h%c%pw = 0
+      h%c%logv = .true.
+                                          
       return
 
 c     ***************************
@@ -264,7 +306,9 @@ c            h%dn(i,j, k) = h%dn(i,j, k)  + 1
 c     ***********************
       entry kwhists3( h, inorm )
 c     ************* take statistics
-      h%norm = inorm
+      if( inorm .ne. -1.0) then
+         h%c%norm = inorm
+      endif
 
       h%x%imin = h%x%nhist
       h%x%imax = 1
@@ -289,53 +333,74 @@ c     ************* take statistics
             enddo
          enddo
       enddo
-      if( h%norm .eq. 0. .and. sum .gt. 0.) then
-         h%norm = sum
-      endif
-      if( h%norm .eq. 0. ) then
-         h%norm = 1.
-      endif
 
-      dx = h%x%bin      
-      dy = h%y%bin
       dz = h%z%bin
-
       do i = h%x%imin, h%x%imax
-         if(h%x%tklg ) then
-            dx  = 10.0**(h%x%xm + i * h%x%bin) -
-     *           10.0**(h%x%xm + (i-1)*h%x%bin)
-         endif
-
          do j = h%y%imin, h%y%imax
-            if(h%y%tklg ) then
-               dy  = 10.0**(h%y%xm + j * h%y%bin) -
-     *               10.0**(h%y%xm + (j-1)*h%y%bin)
-            endif
-            sum = 0.
-            do k = h%z%imin, h%z%imax
-               sum = sum + h%dnw(i,j,k) 
-            enddo
-            if(inorm .eq. 0. .and. sum .gt. 0.) then
-               h%norm = sum
-            elseif(inorm .eq. 0.) then
-               h%norm = 1.
-            endif
+            call kwhistgetnorm3(h, i, j, dx, dy, sum,  normf)
             do k = h%z%imin, h%z%imax
                if(h%z%tklg ) then
                   dz  = 10.0**(h%z%xm + k * h%z%bin) -
      *                  10.0**(h%z%xm + (k-1)*h%z%bin)
                endif
-               h%dndxdydz(i,j,k) = h%dnw(i,j,k)/dx/dy/dz/h%norm
+               h%dndxdydz(i,j,k) = h%dnw(i,j,k)/dx/dy/dz/normf
             enddo
          enddo
       enddo
       return
+c     ****************
+      entry kwhistev3(h, evno)
+c     ********************
+      h%c%eventno = evno
+      return
+c                        
 c     *********************
       entry kwhistid3( h, id)
 c     ********************
-      h%id = id
+      h%c%id = id
       return
+c     ********************
+      entry kwhistai3(h,  title, categ, dNunit,
+     *                logv, pw, labelx, unitx,
+     *                labely, unity, labelz, unitz )
+c     *******************
+c       additional info.
 
+      h%c%title = title
+      h%c%categ =  categ
+      h%c%dNunit = dNunit
+      h%x%label = labelx
+      h%x%unit =  unitx
+      h%y%label = labely
+      h%y%unit =  unity
+      h%z%label = labelz
+      h%z%unit =  unitz
+      h%c%pw = pw
+      h%c%logv = logv
+      return
+c     *******************
+      entry kwhistdir3(h, dir)
+c     ******************must be called after kwhistai is called
+      if( klena(h%c%categ) .gt. 0 )  then
+         h%c%dir=h%c%categ(1:klena(h%c%categ))//"/"
+     *    //dir(1:klena(dir))
+      else
+         h%c%dir=dir(1:klena(dir))
+      endif
+      return
+c     ********************
+      entry kwhiststep3(h, xstep, ystep)
+c     ******************
+c        give step size of indexes for parameter X and Y
+c
+      if(xstep .gt. 0) then 
+         h%x%step = xstep
+      endif
+      if(ystep .gt. 0) then 
+         h%y%step = ystep
+      endif
+      return
+                                                            
 c     *********************
       entry kwhistpr3( h, fno )
 c     ****************print  hist
@@ -346,40 +411,93 @@ c
       else
          xx = h%x%xm + h%x%bin/2  + h%x%inc*(h%x%imin-1)
       endif
-c       header
-      if( fno .lt. 0 ) then
-         write(*, '(a, 2i4,i5, 4i4, 2i5, g14.4, a)')
-     *    '#hist3 ', h%x%nhist, h%y%nhist, h%z%nhist,
-     *    h%x%imin, h%x%imax, 
-     *    h%y%imin, h%y%imax,
-     *    h%z%imin, h%z%imax, h%norm, h%id(1:klena(h%id))
-      else
-         write(fno, '(a, 2i4,i5, 4i4, 2i5, g14.4, a)')
-     *    '#hist3 ', h%x%nhist, h%y%nhist, h%z%nhist,
-     *    h%x%imin, h%x%imax, 
-     *    h%y%imin, h%y%imax,
-     *    h%z%imin, h%z%imax, h%norm, h%id(1:klena(h%id))
-      endif
-      do i = h%x%imin, h%x%imax
+      do i = h%x%imin, h%x%imax, h%x%step
          if(h%y%tklg ) then
             yy = 10.**(h%y%xm + h%y%bin/2) * h%y%inc**(h%y%imin-1)
          else
             yy = h%y%xm + h%y%bin/2 + h%y%inc*(h%y%imin-1)
          endif
-
-         do j = h%y%imin, h%y%imax
+         do j = h%y%imin, h%y%imax, h%y%step
             if(h%z%tklg ) then
-               zz =10.0**( h%z%xm + h%z%bin/2) * h%z%inc**(h%z%imin-1)
+               zz =10.0**( h%z%xm + h%z%bin/2) *
+     *                h%z%inc**(h%z%imin-1)
             else
                zz = h%z%xmin + h%z%bin/2 + h%z%inc*(h%z%imin-1)
             endif
+c              header
+            call kwhistgetnorm3(h, i, j, dx, dy, sum, normf)
+            if( fno .lt. 0 ) then
+               write(*, '(a,i3)') '#hist3 ', h%c%eventno
+               write(*, '(a,a)') '#t ', h%c%title(1:klena(h%c%title))
+               write(*, '(a,a)') '#c ', h%c%categ(1:klena(h%c%categ))
+               write(*, '(a, a,1x, a)')  '#x ', h%x%label, h%x%unit
+               write(*, '(a, a,1x, a)')  '#y ', h%y%label, h%y%unit
+               write(*, '(a, a,1x, a)')  '#z ', h%z%label, h%z%unit
+               write(*, '(a,f10.2)') '#pw ', h%c%pw
+               write(*, '(a,a)') '#dN ', 
+     *            h%c%dNunit(1:klena(h%c%dNunit))
+               write(*, '(a,a,1p2E11.3)') '#k ', 
+     *             h%c%id(1:klena(h%c%id)), xx, yy
+               itempz  = 0
+               if( h%z%tklg )  itempz = 1
+               itempv = 0
+               if( h%c%logv) itempv = 1
+               write(*,'(a,2i3)')'#l ',  itempz, itempv
+               write(*,'(a, 1p4E11.3)') "#n ", sum,  normf, dx, dy
+            else
+               write(fno, '(a,i3)') '#hist3 ',  h%c%eventno
+               write(fno, '(a,a)') '#t ', h%c%title(1:klena(h%c%title))
+               write(fno, '(a,a)') '#c ', h%c%categ(1:klena(h%c%categ))
+               write(fno, '(a, a,1x, a)')  '#x ', h%x%label, h%x%unit
+               write(fno, '(a, a,1x, a)')  '#y ', h%y%label, h%y%unit
+               write(fno, '(a, a,1x, a)')  '#z ', h%z%label, h%z%unit
+               write(fno, '(a,f10.2)') '#pw ', h%c%pw
+               write(fno, '(a,a)') '#dN ',
+     *            h%c%dNunit(1:klena(h%c%dNunit))
+               write(fno, '(a,a,1p2E11.3)') '#k ', 
+     *             h%c%id(1:klena(h%c%id)), xx, yy
+               itempz  = 0
+               if( h%z%tklg )  itempz = 1
+               itempv = 0
+               if( h%c%logv) itempv = 1
+               write(fno,'(a,2i3)')'#l ',  itempz, itempv
+               write(fno,'(a, 1p4E11.3)') "#n ", sum,  normf, dx, dy
+            endif
+
+            if(fno .lt. 0 ) then
+               write(dirstr,'(a,i2)')
+     *         h%c%dir(1:klena(h%c%dir))//h%x%label,i
+               call kseblk(dirstr,"|", nchar)
+               write(*,'(a,a)') "#d ",dirstr(1:klena(dirstr))
+
+               write(filen, '(a, i3, a)') h%y%label, j, ".dat"
+               call kseblk(filen, "|", nchar)
+               write(*,'(a, a)')   '#f ',  filen(1:klena(filen))
+            else
+               write(dirstr,'(a,i2)')
+     *         h%c%dir(1:klena(h%c%dir))//h%x%label,i
+               call kseblk(dirstr,"|", nchar)
+               write(fno,'(a,a)') "#d ",dirstr(1:klena(dirstr))
+
+               write(filen, '(a, i3, a)') h%y%label, j, ".dat"
+               call kseblk(filen, "|", nchar)
+               write(fno,'(a, a)')   '#f ',  filen(1:klena(filen))
+            endif
+
+            dz = h%z%bin
             do k = h%z%imin, h%z%imax
+               if(h%z%tklg ) then
+                  dz  = 10.0**(h%z%xm + k * h%z%bin) -
+     *                 10.0**(h%z%xm + (k-1)*h%z%bin)
+               endif
                if(fno .lt. 0 ) then
-                  write(*, '(2i4,i5, 5g14.4)') i, j, k,
-     *            xx, yy, zz,  h%dndxdydz(i,j,k), h%dnw(i,j,k)
+                  write(*, '(2i4,i5, 1p6E11.3)')
+     *             i, j, k,
+     *            xx, yy, zz,  h%dndxdydz(i,j,k), h%dnw(i,j,k), dz
                else
-                  write(fno, '(2i4, i5, 5g14.4)')  i, j, k,
-     *           xx, yy, zz,  h%dndxdydz(i,j,k), h%dnw(i,j,k)
+                  write(fno, '(2i4, i5, 1p6E11.3)')
+     *            i, j, k,
+     *             xx, yy, zz,  h%dndxdydz(i,j,k), h%dnw(i,j,k), dz
                endif               
                if( h%z%tklg ) then
                   zz =  zz * h%z%inc 
@@ -388,33 +506,36 @@ c       header
                endif
             enddo
             if(fno .lt. 0) then
-               write(*, '(8i3)')  i, j, 0, 0, 0, 0, 0, 0 
+               write(*, '(9i3)')  0, 0, 0, 0, 0, 0, 0, 0, 0
             else
-               write(fno, '(8i3)')  i, j, 0, 0, 0, 0, 0, 0 
-            endif
-            if( h%y%tklg ) then
-               yy =  yy * h%y%inc 
-            else
-               yy =  yy + h%y%inc
+               write(fno, '(9i3)')  0, 0, 0, 0, 0, 0, 0, 0, 0  
             endif
+            do istep = 1, h%y%step
+               if( h%y%tklg ) then
+                  yy =  yy * h%y%inc 
+               else
+                  yy =  yy + h%y%inc
+               endif
+            enddo
          enddo
 
          if( fno .lt. 0 ) then
-            write(*, '(8i3)')  i,  0, 0, 0, 0, 0, 0, 0 
+            write(*, '(9i3)')  0,  0, 0, 0, 0, 0, 0, 0, 0
          else
-            write(fno,'(8i3)')  i,  0, 0, 0, 0, 0, 0, 0 
-         endif
-
-         if( h%x%tklg ) then
-            xx =  xx * h%x%inc 
-         else
-            xx = xx + h%x%inc
+            write(fno,'(9i3)')  0,  0, 0, 0, 0, 0, 0, 0, 0 
          endif
+         do istep = 1, h%x%step
+            if( h%x%tklg ) then
+               xx =  xx * h%x%inc 
+            else
+               xx = xx + h%x%inc
+            endif
+         enddo
       enddo
       if( fno .lt. 0 ) then
-         write(*, '(8i3)') 0,  0, 0, 0, 0, 0, 0, 0 
+         write(*, '(9i3)') 0,  0, 0, 0, 0, 0, 0, 0, 0 
       else
-         write(fno,'(8i3)') 0,  0, 0, 0, 0, 0, 0, 0 
+         write(fno,'(9i3)') 0,  0, 0, 0, 0, 0, 0, 0, 0 
       endif
       return
 c     *********************
@@ -423,7 +544,7 @@ c     *****************
 c       binary write of h to bfnow
       write(bfnow) '#hist3'
       write(bfnow) h%x%nhist, h%y%nhist, h%z%nhist 
-      write(bfnow) h%x, h%y, h%z, h%norm, h%init, h%id
+      write(bfnow) h%x, h%y, h%z, h%c
       write(bfnow) h%dnw,  h%dndxdydz
       return
 
@@ -435,7 +556,7 @@ c        #hist3 must be read outside
       read(bfnor, end =222) xnbin, ynbin, znbin
       allocate( h%dnw(xnbin, ynbin, znbin) )
       allocate( h%dndxdydz(xnbin, ynbin, znbin) )
-      read(bfnor, end=222)  h%x, h%y, h%z, h%norm, h%init, h%id
+      read(bfnor, end=222)  h%x, h%y, h%z, h%c
       read(bfnor, end= 222) h%dnw, h%dndxdydz
       icon = 0
       return
@@ -447,7 +568,7 @@ c     *****************
       entry kwhistd3(h)
 c     ***************
 c       deallocate histo area
-      h%init = ' '
+      h%c%init = ' '
       deallocate(h%dnw,h%dndxdydz, stat=dealloc)
       if(dealloc .ne. 0) then
          write(0,*) ' failed to deallocated hist 3=',dealloc
@@ -468,21 +589,19 @@ c
      *    ' h1 and h2 diff. size histogram in kwhista3'
          stop 9876
       endif
-      if( h%init .ne. 'initend') then
+      if( h%c%init .ne. 'initend') then
 c           not yet initialized.
          xnbin = h1%x%nhist
          ynbin = h1%y%nhist
          znbin = h1%z%nhist
          allocate( h%dnw(xnbin, ynbin, znbin) )
          allocate( h%dndxdydz(xnbin, ynbin, znbin) )
-         h%init = 'initend'
+         h%c%init = 'initend'
       endif
       h%x = h1%x
       h%y = h1%y
       h%z = h1%z
-      h%norm= h1%norm
-
-      h%id = h1%id
+      h%c = h1%c
       do i = 1, h%x%nhist
          do j = 1, h%y%nhist
             do k = 1, h%z%nhist
@@ -494,10 +613,11 @@ c           not yet initialized.
       end
       subroutine kwhistp3( h, fno )
       implicit none
-      include "Z90histCom.f"
+      include "Z90histCom.h"
 c     ******************
-      include "Z90histc.f"
-      include "Z90hist3.f"
+      include "Z90histc.h"
+      include "Z90histo.h"
+      include "Z90hist3.h"
       type(histogram3) h
 
       integer fno
@@ -508,4 +628,51 @@ c     ******************
          call kwhistw3( h, fno )
       endif
       end
+      subroutine kwhistgetnorm3(h, i, j,  dx, dy,  sum, normf)
+c     ******************
+      include 'Z90histc.h'
+      include "Z90histo.h"
+      include "Z90hist3.h"
+      type(histogram3) h
+c     ====================
+      integer i, j                 ! input. index of x for histogram h(x,y)
+      real normf                ! output. normalizaton factor
+      real dx, dy               !  //     bin of x
+      
+      integer k
+      real  sum
 
+      dx = h%x%bin
+      dy = h%y%bin
+      if(h%x%tklg ) then
+         dx  = 10.0**(h%x%xm + i * h%x%bin) -
+     *           10.0**(h%x%xm + (i-1)*h%x%bin)
+      endif
+         
+      
+      if(h%y%tklg ) then
+         dy  = 10.0**(h%y%xm + j * h%y%bin) -
+     *        10.0**(h%y%xm + (j-1)*h%y%bin)
+      endif
+                                                                                                                            
+
+      sum = 0.
+      do k = h%z%imin, h%z%imax
+         sum = sum + h%dnw(i,j,k)
+      enddo
+      if( sum .eq. 0. ) then
+         sum = 1.
+      else 
+         sum = sum/dx/dy
+      endif
+
+      if(h%c%norm .eq. 0.) then
+         normf = sum
+      elseif(h%c%norm .lt. 0.) then
+         normf = 1.0  ! strange. not initialize
+      else
+         normf = h%c%norm
+      endif
+      end
+                                                            
+                                                
