@@ -130,9 +130,10 @@ c        check if in the ascending order of detph
        if(error) then
           call cerrorMsg('Obsev. depth/height not in order', 0)
        endif
-       if(ObsPlane .eq. spherical .and. Za1ry .ne. 'is') then
+       if(ObsPlane .eq. spherical .and.
+     *       (Za1ry .ne. 'is' .and. Za1ry .ne. 'cos' ) ) then
           call cerrorMsg(
-     *      "ObsPlane==spherical but Za1ry !='is'",0)
+     *      "ObsPlane==spherical but Za1ry !='is'/'cos'",0)
        endif
 c       from v5.66
        if(BorderHeightL .eq. 0.) then
